# standby.ps1
# For use with Stelo Data Replicator (SQDR)
# Copyright 2023 StarQuest Ventures, Inc. 

# For instructions, see technical document <TBA>

# First run primary.ps1 on the Primary system
# This will create a backup of the database; copy the backup to $home on this system
# Run this script (standby.ps1) to restore the database, configure the standby and start HADR as standby.
# Return to the primary system and issue the command db2 START HADR ON DB $dbname AS PRIMARY

# The only parameters that need to be supplied are hadr_local_host, hadr_remote_host, and dbname
# You will be prompted to supply these values, or you can supply them on the command line e.g.
#   ./primary.ps1 -hadr_local_host "myhostname" -hadr_remote_host "rmthostname" -dbname "SQDRC"

param ( 
$hadr_local_host = $( Read-Host  "Input local hostname or IP address" ),
$hadr_remote_host = $( Read-Host  "Input remote hostname or IP address" ),
$dbname = $( Read-Host  "Input database name e.g. SQDRC, SQDRP0" ), 
$archivedir='L:\db2archivedir', 
$hadr_local_svc = 0, 
$hadr_remote_svc = 0,
$hadr_remote_inst = $env:DB2INSTANCE,
$hadr_syncmode = 'NEARSYNC',
$hadr_timeout = 120,
$hadr_peer_window = 300
)



# change some of the defaults for Linux environment
if ($IsLinux) {
    Write-Host "Linux"
    if ($archivedir = 'L:\db2archivedir')
    {
	$archivedir = '/home/db2inst1/archivedir'
    }
}

# If the user has not specified the local and remote ports (i.e. port values are still 0)
#  then calculate ports based on the database name - e.g. SQDRP5 will use 50505 
# Also test that the ports are not already in use.

# The range 50500-50599 was chosen because
#   it is below 51000, the start of the dynamic port range configured by the SQDR Plus installer for Windows
#   it is well above 50000 (Db2 LUW listener) and 50005 etc (used by SQDR Plus IPC)
# We can use the same port for both local and remote

$local_svc_base = 50500
$remote_svc_base = 50500

if ( ($hadr_local_svc -eq 0) -and ($hadr_remote_svc -eq 0) )
{
	write-host "Calculating ports based on database name $dbname"

# For automatic port calculation, the database name must be either SQDRC or SQDRPn
	if ($dbname.Length -lt 5)
	{
		write-host "$dbname is not an SQDR database; you must specify hadr_local_svc and hadr_remote_svc"
		write-host "Database name must be at least 5 characters"
		exit	
	}

	if ([string]$dbname.Substring(0,4) -ne "SQDR") 
	{
		write-host "$dbname is not an SQDR database; you must specify hadr_local_svc and hadr_remote_svc"
		write-host "Database name must start with SQDR"
		exit
	}

	if ($dbname -eq "SQDRC") 
	{	
		$n = 99
	}
	else
	{
		$n = $dbname.Substring(5)
		if ($n -notmatch '^\d+$')
		{
			write-host "$dbname is not an SQDR database; you must specify hadr_local_svc and hadr_remote_svc"
			write-host "$n is not an integer"
		write-host $local_svc_base + $n
			exit
		}
	}
	$hadr_local_svc = $local_svc_base + $n
	$hadr_remote_svc = $remote_svc_base + $n
}


write-host Using ports $hadr_local_svc and $hadr_remote_svc

$hadr_target_list="${hadr_remote_host}:${hadr_remote_svc}"
# write-host $hadr_target_list

# Verify that the local port is not already in use

$results = Get-NetTCPConnection | where Localport -eq $hadr_local_svc
if ($results.Length -ne 0)
{
	write-host "Local port $hadr_local_svc is already in use"
	exit
}

# We do not test the remote port since the Primary may already be configured



set-item -path env:DB2CLP -value "**$$**"


if (-Not (Test-Path -Path $archivedir)) {
    mkdir $archivedir
}

db2 RESTORE DB $dbname FROM $home

write-host "db2 UPDATE DB CFG FOR $dbname USING HADR_LOCAL_HOST  $hadr_local_host"
db2 UPDATE DB CFG FOR $dbname USING HADR_LOCAL_HOST  $hadr_local_host
write-host "db2 UPDATE DB CFG FOR $dbname USING HADR_LOCAL_SVC   $hadr_local_svc"
db2 UPDATE DB CFG FOR $dbname USING HADR_LOCAL_SVC   $hadr_local_svc
write-host "db2 UPDATE DB CFG FOR $dbname USING HADR_REMOTE_HOST $hadr_remote_host"
db2 UPDATE DB CFG FOR $dbname USING HADR_REMOTE_HOST $hadr_remote_host
write-host "db2 UPDATE DB CFG FOR $dbname USING HADR_REMOTE_SVC  $hadr_remote_svc"
db2 UPDATE DB CFG FOR $dbname USING HADR_REMOTE_SVC  $hadr_remote_svc
write-host "db2 UPDATE DB CFG FOR $dbname USING HADR_REMOTE_INST $hadr_remote_inst"
db2 UPDATE DB CFG FOR $dbname USING HADR_REMOTE_INST $hadr_remote_inst
write-host "db2 UPDATE DB CFG FOR $dbname USING HADR_SYNCMODE    $hadr_syncmode"
db2 UPDATE DB CFG FOR $dbname USING HADR_SYNCMODE    $hadr_syncmode
write-host "db2 UPDATE DB CFG FOR $dbname USING HADR_TIMEOUT     $hadr_timeout"
db2 UPDATE DB CFG FOR $dbname USING HADR_TIMEOUT     $hadr_timeout
write-host "db2 UPDATE DB CFG FOR $dbname USING HADR_PEER_WINDOW $hadr_peer_window"
db2 UPDATE DB CFG FOR $dbname USING HADR_PEER_WINDOW $hadr_peer_window
write-host "db2 UPDATE DB CFG FOR $dbname USING HADR_TARGET_LIST $hadr_target_list"
db2 UPDATE DB CFG FOR $dbname USING HADR_TARGET_LIST $hadr_target_list

db2 deactivate db $dbname
db2 START HADR ON DB $dbname AS STANDBY

write-host "Standby setup done."
write-host "Now return to the primary system and invoke "
write-host " db2 START HADR ON DB $dbname AS PRIMARY"
write-host "Invoke db2pd -db $dbname -hadr" from an elevated Db2 command window to view the status of HADR.
